@@ -1,8 +1,8 @@
-/* Time-stamp: <2007-06-01 23:03:58 jcs>
+/*
 |
 |  Copyright (C) 2002-2006 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-| 
+|
 |  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
@@ -24,17 +24,19 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_photoalbum.c 1515 2007-06-01 14:19:21Z jcsjcs $
+|  $Id$
 */
 #include <config.h>
 
 #include "itdb_private.h"
 #include "itdb_device.h"
 #include "db-artwork-parser.h"
+#include "db-image-parser.h"
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
 #ifdef HAVE_GDKPIXBUF
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
@@ -84,7 +86,7 @@
 
    For example, "MA450" would stand for an 80 GB 6th generation iPod
    Video. See itdb_device.c for a list of supported models.
-   
+
    This information will be written to the iPod when the PhotoDB is
    saved (itdb_device_write_sysinfo() is called).
 */
@@ -112,7 +114,6 @@ static void error_no_photos_dir (const gchar *mp, GError **error)
     }
 }
 
-
 /**
  * itdb_get_photos_dir:
  * @mountpoint: mountpoint of iPod
@@ -120,8 +121,10 @@ static void error_no_photos_dir (const gchar *mp, GError **error)
  * Retrieve the Photo directory by
  * first calling itdb_get_control_dir() and then adding 'Photos'
  *
- * Return value: path to the Artwork directory or NULL of
+ * Returns: path to the Artwork directory or NULL if
  * non-existent. Must g_free() after use.
+ *
+ * Since: 0.4.0
  */
 gchar *itdb_get_photos_dir (const gchar *mountpoint)
 {
@@ -136,6 +139,7 @@ gchar *itdb_get_photos_dir (const gchar *mountpoint)
 
     for (ptr=paths; *ptr && !result; ++ptr)
     {
+        g_free (result);
 	result = itdb_resolve_path (mountpoint, (const gchar **)*ptr);
     }
     return result;
@@ -147,9 +151,11 @@ gchar *itdb_get_photos_dir (const gchar *mountpoint)
  *
  * Retrieve a path to the Photo DB
  *
- * Return value: path to the PhotoDB or NULL if non-existent. Must
+ * Returns: path to the PhotoDB or NULL if non-existent. Must
  * g_free() after use.
- **/
+ *
+ * Since: 0.4.0
+ */
 gchar *itdb_get_photodb_path (const gchar *mountpoint)
 {
     gchar *photo_dir, *path=NULL;
@@ -174,8 +180,10 @@ gchar *itdb_get_photodb_path (const gchar *mountpoint)
  * Retrieve the Photo Thumbnail directory by
  * first calling itdb_get_control_dir() and then adding 'Photos/Thumbs'
  *
- * Return value: path to the Artwork directory or NULL of
+ * Returns: path to the Artwork directory or NULL if
  * non-existent. Must g_free() after use.
+ *
+ * Since: 0.4.0
  */
 gchar *itdb_get_photos_thumb_dir (const gchar *mountpoint)
 {
@@ -197,16 +205,17 @@ gchar *itdb_get_photos_thumb_dir (const gchar *mountpoint)
     return result;
 }
 
-
 /**
  * itdb_photodb_parse:
- * @mp: mountpoint of the iPod
- * @error: will contain the error description when an error occured.
+ * @mp:     mountpoint of the iPod
+ * @error:  will contain the error description when an error occured.
  *
  * Parses the photo database of an iPod mounted at @mp.
  *
- * Return value: the imported PhotoDB or NULL in case of an error.
- **/
+ * Returns: the imported PhotoDB or NULL in case of an error.
+ *
+ * Since: 0.4.0
+ */
 Itdb_PhotoDB *itdb_photodb_parse (const gchar *mp, GError **error)
 {
     gchar *photos_dir;
@@ -236,7 +245,6 @@ Itdb_PhotoDB *itdb_photodb_parse (const gchar *mp, GError **error)
     return photodb;
 }
 
-
 /**
  * itdb_photodb_create:
  * @mountpoint: mountpoint or NULL.
@@ -244,10 +252,12 @@ Itdb_PhotoDB *itdb_photodb_parse (const gchar *mp, GError **error)
  * Creates a new Itdb_PhotoDB. If mountpoint is NULL, you will have to
  * set it manually later by calling itdb_device_set_mountpoint().
  *
- * Return value: a newly created Itdb_PhotoDB to be freed with
+ * Returns: a newly created Itdb_PhotoDB to be freed with
  * itdb_photodb_free() when it's no longer needed. The Photo Library
  * Album is created automatically.
- **/
+ *
+ * Since: 0.4.2
+ */
 Itdb_PhotoDB *itdb_photodb_create (const gchar *mountpoint)
 {
     Itdb_PhotoDB *photodb = itdb_photodb_new ();
@@ -265,7 +275,6 @@ Itdb_PhotoDB *itdb_photodb_create (const gchar *mountpoint)
 }
 
 
-
 static Itdb_PhotoDB *itdb_photodb_new (void)
 {
     Itdb_PhotoDB *photodb;
@@ -276,14 +285,14 @@ static Itdb_PhotoDB *itdb_photodb_new (void)
     return photodb;
 }
 
-
-
-/** 
+/**
  * itdb_photodb_free:
  * @photodb: an #Itdb_PhotoDB
  *
- * Free the memory taken by @photodb. 
- **/
+ * Free the memory taken by @photodb.
+ *
+ * Since: 0.4.0
+ */
 void itdb_photodb_free (Itdb_PhotoDB *photodb)
 {
 	if (photodb)
@@ -306,25 +315,33 @@ void itdb_photodb_free (Itdb_PhotoDB *photodb)
 
 
 
-G_GNUC_INTERNAL gint itdb_get_free_photo_id ( Itdb_PhotoDB *db ) 
+G_GNUC_INTERNAL gint itdb_get_max_photo_id ( Itdb_PhotoDB *db )
 {
-	gint photo_id = 0;
+	gint max_seen_id = 0;
 	GList *it;
 
 	for (it = db->photos; it != NULL; it = it->next) {
 		Itdb_Artwork *artwork;
 
 		artwork = (Itdb_Artwork *)it->data;
-		if( artwork->id > photo_id )
-			photo_id = artwork->id;
+		if( artwork->id > max_seen_id )
+			max_seen_id = artwork->id;
 	}
-	return photo_id + 1;
+        for (it = db->photoalbums; it != NULL; it = it->next) {
+		Itdb_PhotoAlbum *album;
+		album = (Itdb_PhotoAlbum *)it->data;
+		if ( album->album_id > max_seen_id )
+			max_seen_id = album->album_id;
+		
+	}
+	return max_seen_id;
 }
 
 static void itdb_photodb_photoalbum_free (Itdb_PhotoAlbum *album)
 {
     if (album)
     {
+	album->photodb = NULL;
 	g_free (album->name);
 	g_list_free (album->members);
 
@@ -348,10 +365,9 @@ static Itdb_Artwork *itdb_photodb_add_photo_internal (Itdb_PhotoDB *db,
 						      GError **error)
 {
 #ifdef HAVE_GDKPIXBUF
-    gboolean result;
+    gboolean result=FALSE;
     Itdb_Artwork *artwork;
     Itdb_PhotoAlbum *album;
-    const Itdb_ArtworkFormat *format;
 
     g_return_val_if_fail (db, NULL);
     g_return_val_if_fail (db->device, NULL);
@@ -359,7 +375,7 @@ static Itdb_Artwork *itdb_photodb_add_photo_internal (Itdb_PhotoDB *db,
     g_return_val_if_fail (!(image_data && (image_data_len == 0)), NULL);
     g_return_val_if_fail (!(pixbuf && (!GDK_IS_PIXBUF (pixbuf))), NULL);
 
-    if (!ipod_supports_photos (db->device))
+    if (!itdb_device_supports_photo (db->device))
     {
 	const Itdb_IpodInfo *ipodinfo = itdb_device_get_ipod_info (db->device);
 	const gchar *model, *generation;
@@ -408,40 +424,21 @@ static Itdb_Artwork *itdb_photodb_add_photo_internal (Itdb_PhotoDB *db,
 
     artwork = itdb_artwork_new ();
 
-    /* Add a thumbnail for every supported format */
-    format = itdb_device_get_artwork_formats (db->device);
-    g_return_val_if_fail (format, NULL);
-
-    for(result = TRUE; format->type != -1 && result == TRUE; format++)
+    if (filename)
     {
-	if((format->type == ITDB_THUMB_COVER_SMALL) ||
-	   (format->type == ITDB_THUMB_COVER_LARGE))
-	    continue;
-	if (filename)
-	{
-	    result = itdb_artwork_add_thumbnail (artwork,
-						 format->type,
-						 filename,
-						 rotation,
-						 error);
-	}
-	if (image_data)
-	{
-	    result = itdb_artwork_add_thumbnail_from_data (artwork,
-							   format->type,
-							   image_data,
-							   image_data_len,
-							   rotation,
-							   error);
-	}
-	if (pixbuf) 
-	{
-	  result = itdb_artwork_add_thumbnail_from_pixbuf (artwork,
-							   format->type, 
-							   pixbuf,
-							   rotation,
-							   error);
-	}
+        result = itdb_artwork_set_thumbnail (artwork, filename,
+                                             rotation, error);
+    }
+    if (image_data)
+    {
+        result = itdb_artwork_set_thumbnail_from_data (artwork, image_data,
+                                                       image_data_len,
+                                                       rotation, error);
+    }
+    if (pixbuf) 
+    {
+        result = itdb_artwork_set_thumbnail_from_pixbuf (artwork, pixbuf,
+                                                         rotation, error);
     }
 
     if (result != TRUE)
@@ -473,28 +470,28 @@ static Itdb_Artwork *itdb_photodb_add_photo_internal (Itdb_PhotoDB *db,
 #endif
 }
 
-
-
 /**
  * itdb_photodb_add_photo:
- * @db: the #Itdb_PhotoDB to add the photo to.
- * @filename: file with the photo to add.
- * @position: position where to insert the new photo (-1 to append at
- * the end)
- * @rotation: angle by which the image should be rotated
- * counterclockwise. Valid values are 0, 90, 180 and 270.
- * @error: return location for a #GError or NULL
- * 
+ * @db:         the #Itdb_PhotoDB to add the photo to
+ * @filename:   path of the photo to add.
+ * @position:   position where to insert the new photo (-1 to append
+ *              at the end)
+ * @rotation:   angle by which the image should be rotated
+ *              counterclockwise. Valid values are 0, 90, 180 and 270.
+ * @error:      return location for a #GError or NULL
+ *
  * Add a photo to the PhotoDB. The photo is automatically added to the
  * first Photoalbum, which by default contains a list of all photos in
  * the database. If no Photoalbums exist one is created automatically.
  *
  * For the rotation angle you can also use the gdk constants
- * GDK_PIXBUF_ROTATE_NONE, ..._COUNTERCLOCKWISE, ..._UPSIDEDOWN AND
- * ..._CLOCKWISE.
+ * %GDK_PIXBUF_ROTATE_NONE, %GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE,
+ * %GDK_PIXBUF_ROTATE_UPSIDEDOWN, AND %GDK_PIXBUF_ROTATE_CLOCKWISE.
  *
- * Return value: a pointer to the added photo.
- **/
+ * Returns: a pointer to the added photo.
+ *
+ * Since: 0.4.0
+ */
 Itdb_Artwork *itdb_photodb_add_photo (Itdb_PhotoDB *db,
 				      const gchar *filename,
 				      gint position,
@@ -508,29 +505,30 @@ Itdb_Artwork *itdb_photodb_add_photo (Itdb_PhotoDB *db,
 					    position, rotation, error);
 }
 
-
 /**
  * itdb_photodb_add_photo_from_data:
- * @db: the #Itdb_PhotoDB to add the photo to.
- * @image_data: chunk of memory containing the image data (for example
- * a jpg file)
+ * @db:             the #Itdb_PhotoDB to add the photo to
+ * @image_data:     chunk of memory containing the image data (for
+ *                  example a jpg file)
  * @image_data_len: length of above chunk of memory
- * @position: position where to insert the new photo (-1 to append at
- * the end)
- * @rotation: angle by which the image should be rotated
- * counterclockwise. Valid values are 0, 90, 180 and 270.
- * @error: return location for a #GError or NULL
- * 
+ * @position:       position where to insert the new photo (-1 to
+ *                  append at the end)
+ * @rotation:       angle by which the image should be rotated
+ *                  counterclockwise. Valid values are 0, 90, 180 and 270.
+ * @error:          return location for a #GError or NULL
+ *
  * Add a photo to the PhotoDB. The photo is automatically added to the
  * first Photoalbum, which by default contains a list of all photos in
  * the database. If no Photoalbums exist one is created automatically.
  *
  * For the rotation angle you can also use the gdk constants
- * GDK_PIXBUF_ROTATE_NONE, ..._COUNTERCLOCKWISE, ..._UPSIDEDOWN AND
- * ..._CLOCKWISE.
+ * %GDK_PIXBUF_ROTATE_NONE, %GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE,
+ * %GDK_PIXBUF_ROTATE_UPSIDEDOWN, AND %GDK_PIXBUF_ROTATE_CLOCKWISE.
  *
- * Return value: a pointer to the added photo.
- **/
+ * Returns: a pointer to the added photo.
+ *
+ * Since: 0.4.0
+ */
 Itdb_Artwork *itdb_photodb_add_photo_from_data (Itdb_PhotoDB *db,
 						const guchar *image_data,
 						gsize image_data_len,
@@ -548,24 +546,26 @@ Itdb_Artwork *itdb_photodb_add_photo_from_data (Itdb_PhotoDB *db,
 
 /**
  * itdb_photodb_add_photo_from_pixbuf:
- * @db: the #Itdb_PhotoDB to add the photo to.
- * @pixbuf: a #GdkPixbuf to use as the image data
- * @position: position where to insert the new photo (-1 to append at
- * the end)
- * @rotation: angle by which the image should be rotated
- * counterclockwise. Valid values are 0, 90, 180 and 270.
- * @error: return location for a #GError or NULL
- * 
+ * @db:         the #Itdb_PhotoDB to add the photo to
+ * @pixbuf:     a #GdkPixbuf to use as the image data
+ * @position:   position where to insert the new photo (-1 to append
+ *              at the end)
+ * @rotation:   angle by which the image should be rotated
+ *              counterclockwise. Valid values are 0, 90, 180 and 270.
+ * @error:      return location for a #GError or NULL
+ *
  * Add a photo to the PhotoDB. The photo is automatically added to the
  * first Photoalbum, which by default contains a list of all photos in
  * the database. If no Photoalbums exist one is created automatically.
  *
  * For the rotation angle you can also use the gdk constants
- * GDK_PIXBUF_ROTATE_NONE, ..._COUNTERCLOCKWISE, ..._UPSIDEDOWN AND
- * ..._CLOCKWISE.
+ * %GDK_PIXBUF_ROTATE_NONE, %GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE,
+ * %GDK_PIXBUF_ROTATE_UPSIDEDOWN, AND %GDK_PIXBUF_ROTATE_CLOCKWISE.
  *
- * Return value: a pointer to the added photo.
- **/
+ * Returns: a pointer to the added photo.
+ *
+ * Since: 0.5.0
+ */
 Itdb_Artwork *itdb_photodb_add_photo_from_pixbuf (Itdb_PhotoDB *db,
 						  gpointer pixbuf,
 						  gint position,
@@ -579,22 +579,25 @@ Itdb_Artwork *itdb_photodb_add_photo_from_pixbuf (Itdb_PhotoDB *db,
 					    position, rotation, error);
 }
 
-
 /**
  * itdb_photodb_remove_photo:
- * @db: the #Itdb_PhotoDB to remove the photo from
- * @album: the album to remove the photo from. If album is NULL, then
- * it will first be removed from all photoalbums and then from the
- * photo database as well.
- * @photo: #Itdb_Artwork (photo) to remove.
+ * @db:     the #Itdb_PhotoDB to remove the photo from
+ * @album:  the album to remove the photo from. If album is NULL, then
+ *          it will first be removed from all photoalbums and then
+ *          from the photo database as well.
+ * @photo:  #Itdb_Artwork (photo) to remove.
  *
- * Remove photo. If @album is not the first photoalbum, the photo will
- * be removed from that album only. If @album is NULL or the first
- * photoalbum (Photo Library), the photo will be removed from all
- * albums and the #Itdb_PhotoDB.
+ * Removes a photo. If @album is not the first photoalbum, the photo
+ * will be removed from that album only. If @album is NULL or the
+ * first photoalbum (Photo Library), the photo will be removed from
+ * all albums and the #Itdb_PhotoDB.
  *
+ * <note>
  * @photo will be freed and can no longer be used if removed from the
  * first photoalbum.
+ * </note>
+ *
+ * Since: 0.4.0
  */
 void itdb_photodb_remove_photo (Itdb_PhotoDB *db,
 				Itdb_PhotoAlbum *album,
@@ -612,10 +615,7 @@ void itdb_photodb_remove_photo (Itdb_PhotoDB *db,
         for (it = db->photoalbums; it != NULL; it = it->next)
 	{
             Itdb_PhotoAlbum *_album = it->data;
-	    while (g_list_find (_album->members, photo))
-	    {
-		_album->members = g_list_remove (_album->members, photo);
-	    }
+            _album->members = g_list_remove_all (_album->members, photo);
         }
         /* Remove the photo from the image list */
 	db->photos = g_list_remove (db->photos, photo);
@@ -631,15 +631,17 @@ void itdb_photodb_remove_photo (Itdb_PhotoDB *db,
 
 /**
  * itdb_photodb_photoalbum_by_name:
- * @db: the #Itdb_PhotoDB to retrieve the album from
- * @albumname: the name of the photoalbum to get or NULL for the
- * master photoalbum.
+ * @db:         the #Itdb_PhotoDB to retrieve the album from
+ * @albumname:  the name of the photoalbum to get or NULL for the
+ *              master photoalbum.
  *
  * Find the first photoalbum with a given name or the Photo Library
  * Album if called with no name.
  *
- * Return value: a pointer to the first photoalbum named @albumname,
+ * Returns: a pointer to the first photoalbum named @albumname,
  * else NULL
+ *
+ * Since: 0.4.2
  */
 Itdb_PhotoAlbum *itdb_photodb_photoalbum_by_name (Itdb_PhotoDB *db, const gchar *albumname)
 {
@@ -660,23 +662,25 @@ Itdb_PhotoAlbum *itdb_photodb_photoalbum_by_name (Itdb_PhotoDB *db, const gchar
 
 /**
  * itdb_photodb_photoalbum_remove:
- * @db: the #Itdb_PhotoDB to apply changes to
- * @album: the album to be removed from the database
- * @remove_pics: TRUE to remove pics in that album permanently from
- * the database.
+ * @db:             the #Itdb_PhotoDB to apply changes to
+ * @album:          the album to be removed from the database
+ * @remove_pics:    TRUE to remove pics in that album permanently
+ *                  from the database.
  *
- * Remove @album from the Photo Database. If remove_pics is TRUE,
+ * Remove @album from the Photo Database. If @remove_pics is TRUE,
  * remove all photos contained in @album from the Photo Database.
  *
+ * <note>
  * Memory used by the removed album will be freed and the album cannot
  * be accessed any more.
- **/
+ * </note>
+ *
+ * Since: 0.4.2
+ */
 void itdb_photodb_photoalbum_remove (Itdb_PhotoDB *db,
 				     Itdb_PhotoAlbum *album,
 				     gboolean remove_pics)
 {
-        GList *it;
-
         g_return_if_fail (db);
         g_return_if_fail (album);
 
@@ -684,11 +688,16 @@ void itdb_photodb_photoalbum_remove (Itdb_PhotoDB *db,
 	 * and remove them from the database */
         if (remove_pics)
 	{
-            for (it = album->members; it != NULL; it = it->next )
+	    /* we can't iterate over album->members because
+	       itdb_photodb_remove_photo() modifies album->members in
+	       a not easily predicable way (e.g. @photo may exist in the
+	       album several times). Therefore we remove photos until
+	       album->members is empty. */
+	    while (album->members)
 	    {
-                Itdb_Artwork *photo = it->data;
-                itdb_photodb_remove_photo (db, NULL, photo);
-            }
+		Itdb_Artwork *photo = album->members->data;
+		itdb_photodb_remove_photo (db, NULL, photo);
+	    }
         }
         db->photoalbums = g_list_remove (db->photoalbums, album);
 	itdb_photodb_photoalbum_free (album);
@@ -696,11 +705,12 @@ void itdb_photodb_photoalbum_remove (Itdb_PhotoDB *db,
 
 /**
  * itdb_photodb_photoalbum_add_photo:
- * @db: the #Itdb_PhotoDB to act on
- * @album: the #Itdb_PhotoAlbum to add the photo to
- * @photo: a pointer to the photo (#Itdb_Artwork) to add to the album
- * @position: position where to insert the new photo (-1 to append at
- * the end)
+ * @db:         the #Itdb_PhotoDB to act on
+ * @album:      the #Itdb_PhotoAlbum to add the photo to
+ * @photo:      a pointer to the photo (#Itdb_Artwork) to add to the
+ *              album
+ * @position:   position where to insert the new photo (-1 to append
+ *              at the end)
  *
  * Adds a photo already in the library to the specified album
  * @album. Photos are automatically added to the first album (Photo
@@ -708,8 +718,8 @@ void itdb_photodb_photoalbum_remove (Itdb_PhotoDB *db,
  * itdb_photodb_add_photo_from_data(), so you don't have to use this
  * function to add them there.
  *
+ * Since: 0.4.2
  */
-
 void itdb_photodb_photoalbum_add_photo (Itdb_PhotoDB *db,
 					Itdb_PhotoAlbum *album,
 					Itdb_Artwork *photo,
@@ -722,17 +732,18 @@ void itdb_photodb_photoalbum_add_photo (Itdb_PhotoDB *db,
     album->members = g_list_insert (album->members, photo, position);
 }
 
-
 /**
  * itdb_photodb_photoalbum_create:
- * @db: The database to create a new album in
- * @albumname: the name of the new album
- * @pos: position where to insert the newly created album (-1 for
- * append to end).
+ * @db:         The database to create a new album in
+ * @albumname:  the name of the new album
+ * @pos:        position where to insert the newly created album (-1
+ *              to append at the end).
  *
  * Create and add a new photoalbum.
  *
- * Return value: the album which was created and added.
+ * Returns: the album which was created and added.
+ *
+ * Since: 0.4.2
  */
 Itdb_PhotoAlbum *itdb_photodb_photoalbum_create (Itdb_PhotoDB *db,
 						 const gchar *albumname,
@@ -745,6 +756,7 @@ Itdb_PhotoAlbum *itdb_photodb_photoalbum_create (Itdb_PhotoDB *db,
 
 	album = g_new0 (Itdb_PhotoAlbum, 1);
 	album->album_type = 2; /* normal album, set to 1 for Photo Library */
+	album->photodb = db;
 	album->name = g_strdup(albumname);
 	db->photoalbums = g_list_insert (db->photoalbums, album, pos);
 
@@ -753,16 +765,18 @@ Itdb_PhotoAlbum *itdb_photodb_photoalbum_create (Itdb_PhotoDB *db,
 
 /**
  * itdb_photodb_write:
- * @photodb: the #Itdb_PhotoDB to write to disk
- * @error: return location for a #GError or NULL
+ * @photodb:    the #Itdb_PhotoDB to write to disk
+ * @error:      return location for a #GError or NULL
  *
  * Write out a PhotoDB.
  *
  * FIXME: error is not set yet.
  *
- * Return value: TRUE on success, FALSE on error, in which case @error is
+ * Returns: TRUE on success, FALSE on error, in which case @error is
  * set accordingly.
- **/
+ *
+ * Since: 0.4.0
+ */
 gboolean itdb_photodb_write (Itdb_PhotoDB *photodb, GError **error)
 {
     gint result;
