@@ -1,8 +1,7 @@
-/* Time-stamp: <2007-02-25 11:52:48 jcs>
-|
+/*
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-| 
+|
 |  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_playlist.c 1423 2007-04-30 00:46:48Z jcsjcs $
+|  $Id$
 */
 
 #include "itdb_private.h"
@@ -40,13 +39,13 @@
 
 /**
  * itdb_spl_action_known:
- * @action: an #SPLAction
+ * @action: an #ItdbSPLAction
  *
  * Checks if @action is a known (to libgpod) smart playlist action.
  *
- * Return value: TRUE if @action is known. Otherwise a warning is displayed 
- * and FALSE is returned. 
- **/
+ * Returns: TRUE if @action is known. Otherwise a warning is
+ * displayed and FALSE is returned.
+ */
 gboolean itdb_spl_action_known (ItdbSPLAction action)
 {
     gboolean result = FALSE;
@@ -72,11 +71,13 @@ gboolean itdb_spl_action_known (ItdbSPLAction action)
     case ITDB_SPLACTION_IS_NOT:
     case ITDB_SPLACTION_DOES_NOT_CONTAIN:
     case ITDB_SPLACTION_BINARY_AND:
+    case ITDB_SPLACTION_BINARY_UNKNOWN1:
+    case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	result = TRUE;
     }
     if (result == FALSE)
     {	/* New action! */
-	g_warning (_("Unknown action (%d) in smart playlist will be ignored.\n"), action);
+	g_warning (_("Unknown action (0x%x) in smart playlist will be ignored.\n"), action);
     }
     return result;
 }
@@ -87,9 +88,9 @@ gboolean itdb_spl_action_known (ItdbSPLAction action)
  *
  * Gets the type of the field of the @splr rule
  *
- * Return value: an #Itdb_SPLFieldType corresponding to @splr field type 
- * (string, int, date, ...)
- **/
+ * Returns: an #Itdb_SPLFieldType corresponding to @splr field
+ * type (string, int, date, ...)
+ */
 ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr)
 {
     g_return_val_if_fail (splr != NULL, ITDB_SPLFT_UNKNOWN);
@@ -106,6 +107,14 @@ ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr)
     case ITDB_SPLFIELD_COMPOSER:
     case ITDB_SPLFIELD_GROUPING:
     case ITDB_SPLFIELD_TVSHOW:
+    case ITDB_SPLFIELD_CATEGORY:
+    case ITDB_SPLFIELD_DESCRIPTION:
+    case ITDB_SPLFIELD_SORT_SONG_NAME:
+    case ITDB_SPLFIELD_SORT_ALBUM:
+    case ITDB_SPLFIELD_SORT_ALBUMARTIST:
+    case ITDB_SPLFIELD_SORT_ARTIST:
+    case ITDB_SPLFIELD_SORT_COMPOSER:
+    case ITDB_SPLFIELD_SORT_TVSHOW:
 	return ITDB_SPLFT_STRING;
     case ITDB_SPLFIELD_BITRATE:
     case ITDB_SPLFIELD_SAMPLE_RATE:
@@ -116,12 +125,15 @@ ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr)
     case ITDB_SPLFIELD_DISC_NUMBER:
     case ITDB_SPLFIELD_BPM:
     case ITDB_SPLFIELD_RATING:
+    case ITDB_SPLFIELD_ALBUM_RATING:
     case ITDB_SPLFIELD_TIME: /* time is the length of the track in
 			   milliseconds */
     case ITDB_SPLFIELD_SEASON_NR:
     case ITDB_SPLFIELD_SKIPCOUNT:
+    case ITDB_SPLFIELD_PODCAST:
 	return ITDB_SPLFT_INT;
     case ITDB_SPLFIELD_COMPILATION:
+    case ITDB_SPLFIELD_PURCHASE:
 	return ITDB_SPLFT_BOOLEAN;
     case ITDB_SPLFIELD_DATE_MODIFIED:
     case ITDB_SPLFIELD_DATE_ADDED:
@@ -136,15 +148,14 @@ ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr)
     return(ITDB_SPLFT_UNKNOWN);
 }
 
-
 /**
  * itdb_splr_get_action_type:
  * @splr: an #Itdb_SPLRule
- * 
+ *
  * Gets the type of the action associated with @splr.
  *
- * Return value: type (range, date, string...) of the action field 
- **/
+ * Returns: type (range, date, string...) of the action field
+ */
 ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 {
     ItdbSPLFieldType fieldType;
@@ -178,6 +189,8 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_IN_THE_LAST:
 	case ITDB_SPLACTION_IS_NOT_IN_THE_LAST:
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	    return ITDB_SPLAT_INVALID;
 	}
 	/* Unknown action type */
@@ -198,6 +211,8 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_IN_THE_RANGE:
 	    return ITDB_SPLAT_RANGE_INT;
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	case ITDB_SPLACTION_IS_STRING:
 	case ITDB_SPLACTION_CONTAINS:
 	case ITDB_SPLACTION_STARTS_WITH:
@@ -242,11 +257,16 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_NOT:
 	case ITDB_SPLACTION_DOES_NOT_CONTAIN:
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	    return ITDB_SPLAT_INVALID;
 	}
     case ITDB_SPLFT_BINARY_AND:
 	switch ((ItdbSPLAction)splr->action)
 	{
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
+            return ITDB_SPLAT_UNKNOWN;
 	case ITDB_SPLACTION_BINARY_AND:
 	    return ITDB_SPLAT_BINARY_AND;
 	case ITDB_SPLACTION_IS_INT:
@@ -297,6 +317,8 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_NOT:
 	case ITDB_SPLACTION_DOES_NOT_CONTAIN:
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	    return ITDB_SPLAT_INVALID;
 	}
 	/* Unknown action type */
@@ -320,16 +342,15 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
  *
  */
 
-
 /**
  * itdb_splr_eval:
- * @splr: an #Itdb_SPLRule
- * @track: an #Itdb_Track
+ * @splr:   an #Itdb_SPLRule
+ * @track:  an #Itdb_Track
  *
- * Evaluates @splr's truth against @track. track-&gt;itdb must be set.
+ * Evaluates @splr's truth against @track. @track->itdb must be set.
  *
- * Return value: TRUE if @track matches @splr, FALSE otherwise.
- **/
+ * Returns: TRUE if @track matches @splr, FALSE otherwise.
+ */
 gboolean itdb_splr_eval (Itdb_SPLRule *splr, Itdb_Track *track)
 {
     ItdbSPLFieldType ft;
@@ -341,7 +362,6 @@ gboolean itdb_splr_eval (Itdb_SPLRule *splr, Itdb_Track *track)
     guint32 datecomp = 0;
     Itdb_Playlist *playcomp = NULL;
     time_t t;
-    guint64 mactime;
 
     g_return_val_if_fail (splr, FALSE);
     g_return_val_if_fail (track, FALSE);
@@ -424,7 +444,7 @@ gboolean itdb_splr_eval (Itdb_SPLRule *splr, Itdb_Track *track)
 	handled = TRUE;
 	break;
     case ITDB_SPLFIELD_TIME:
-	intcomp = track->tracklen/1000;
+	intcomp = track->tracklen;
 	handled = TRUE;
 	break;
     case ITDB_SPLFIELD_COMPILATION:
@@ -568,13 +588,11 @@ gboolean itdb_splr_eval (Itdb_SPLRule *splr, Itdb_Track *track)
 	case ITDB_SPLACTION_IS_IN_THE_LAST:
 	    time (&t);
 	    t += (splr->fromdate * splr->fromunits);
-	    mactime = itdb_time_host_to_mac (t);
-	    return (datecomp > mactime);
+	    return (datecomp > t);
 	case ITDB_SPLACTION_IS_NOT_IN_THE_LAST:
 	    time (&t);
 	    t += (splr->fromdate * splr->fromunits);
-	    mactime = itdb_time_host_to_mac (t);
-	    return (datecomp <= mactime);
+	    return (datecomp <= t);
 	case ITDB_SPLACTION_IS_IN_THE_RANGE:
 	    return ((datecomp <= splr->fromvalue &&
 		     datecomp >= splr->tovalue) ||
@@ -682,13 +700,12 @@ static GList *randomize_glist (GList *list)
     return list;
 }
 
-
 /**
  * itdb_playlist_randomize:
  * @pl: an #Itdb_Playlist to randomize
  *
  * Randomizes @pl
- **/
+ */
 void itdb_playlist_randomize (Itdb_Playlist *pl)
 {
     g_return_if_fail (pl);
@@ -696,16 +713,16 @@ void itdb_playlist_randomize (Itdb_Playlist *pl)
     pl->members = randomize_glist (pl->members);
 }
 
-
 /**
  * itdb_spl_update:
  * @spl: an #Itdb_Playlist
  *
- * Updates the content of the smart playlist @spl (meant to be called if the 
- * tracks stored in the #Itdb_iTunesDB associated with @spl have changed 
- * somehow and you want spl-&gt;members to be accurate with regards to those 
- * changes. Does nothing if @spl isn't a smart playlist.
- **/
+ * Updates the content of the smart playlist @spl (meant to be called
+ * if the tracks stored in the #Itdb_iTunesDB associated with @spl
+ * have changed somehow and you want @spl->members to be accurate
+ * with regards to those changes. Does nothing if @spl isn't a smart
+ * playlist.
+ */
 void itdb_spl_update (Itdb_Playlist *spl)
 {
     GList *gl;
@@ -912,13 +929,12 @@ void itdb_spl_update (Itdb_Playlist *spl)
     }
 }
 
-
 /**
  * itdb_spl_update_all:
  * @itdb: an #Itdb_iTunesDB
  *
  * Updates all smart playlists contained in @itdb
- **/
+ */
 void itdb_spl_update_all (Itdb_iTunesDB *itdb)
 {
     g_return_if_fail (itdb);
@@ -934,14 +950,15 @@ static void spl_update2 (Itdb_Playlist *playlist, gpointer data)
         itdb_spl_update (playlist);
 }
 
-
-/** 
+/**
  * itdb_spl_update_live:
  * @itdb: an #Itdb_iTunesDB
  *
- * Updates all 'live' smart playlists contained in @itdb, ie those which have 
- * the 'live updating' flag set
- **/
+ * Updates all smart playlists contained in @itdb which have the
+ * @liveupdate flag set.
+ *
+ * Since: 0.2.0
+ */
 void itdb_spl_update_live (Itdb_iTunesDB *itdb)
 {
     g_return_if_fail (itdb);
@@ -953,13 +970,12 @@ void itdb_spl_update_live (Itdb_iTunesDB *itdb)
 /* end of code based on Samuel Wood's work */
 /* ------------------------------------------------------------------- */
 
-
 /**
  * itdb_splr_validate:
  * @splr: an #Itdb_SPLRule
  *
- * Validates a rule 
- **/
+ * Validates a smart playlist rule
+ */
 void itdb_splr_validate (Itdb_SPLRule *splr)
 {
     ItdbSPLActionType at;
@@ -1011,13 +1027,12 @@ void itdb_splr_validate (Itdb_SPLRule *splr)
 
 }
 
-
 /**
  * itdb_splr_free:
  * @splr: an #Itdb_SPLRule
  *
- * Frees the memory used by @splr 
- **/
+ * Frees the memory used by @splr
+ */
 void itdb_splr_free (Itdb_SPLRule *splr)
 {
     if (splr)
@@ -1032,9 +1047,9 @@ void itdb_splr_free (Itdb_SPLRule *splr)
  * @pl: an #Itdb_Playlist
  * @splr: an Itdb_SPLRule
  *
- * Removes the smart playlist rule @splr from playlist @pl. The memory used by
- * @splr is freed.
- **/
+ * Removes the smart playlist rule @splr from playlist @pl. The memory
+ * used by @splr is freed.
+ */
 void itdb_splr_remove (Itdb_Playlist *pl, Itdb_SPLRule *splr)
 {
     g_return_if_fail (pl);
@@ -1046,14 +1061,14 @@ void itdb_splr_remove (Itdb_Playlist *pl, Itdb_SPLRule *splr)
 
 /**
  * itdb_splr_add:
- * @pl: an #Itdb_Playlist
- * @splr: an #Itdb_SPLRule
- * @pos: position of the rule
+ * @pl:     an #Itdb_Playlist
+ * @splr:   an #Itdb_SPLRule
+ * @pos:    position of the rule
  *
- * Adds the smart rule @splr to @pl at position @pos. If @pos is -1, @splr gets
- * appended to the end. After this call, @splr memory is managed by @pl, so 
- * you no longer need to call itdb_splr_free()
- **/
+ * Adds the smart rule @splr to @pl at position @pos. If @pos is -1,
+ * @splr gets appended to the end. After this call, @splr memory is
+ * managed by @pl, so you no longer need to call itdb_splr_free()
+ */
 void itdb_splr_add (Itdb_Playlist *pl, Itdb_SPLRule *splr, gint pos)
 {
     g_return_if_fail (pl);
@@ -1063,15 +1078,14 @@ void itdb_splr_add (Itdb_Playlist *pl, Itdb_SPLRule *splr, gint pos)
 					splr, pos);
 }
 
-
 /**
  * itdb_splr_new:
- * 
- * Creates a new default smart rule 
  *
- * Return value: a new #Itdb_SPLRule that must be freed with itdb_splr_free() when 
+ * Creates a new default smart rule
+ *
+ * Returns: a new #Itdb_SPLRule that must be freed with itdb_splr_free() when
  * no longer needed
- **/
+ */
 Itdb_SPLRule *itdb_splr_new (void)
 {
     Itdb_SPLRule *splr = g_new0 (Itdb_SPLRule, 1);
@@ -1088,18 +1102,18 @@ Itdb_SPLRule *itdb_splr_new (void)
     return splr;
 }
 
-
 /**
  * itdb_splr_add_new:
- * @pl: an #Itdb_Playlist
- * @pos: position to insert the rule at
+ * @pl:     an #Itdb_Playlist
+ * @pos:    position to insert the rule at
  *
- * Creates a new smart rule and inserts it at position @pos in @pl. If @pos is
- * -1, the new rule gets appended to the end.
+ * Creates a new smart rule and inserts it at position @pos in @pl. If
+ * @pos is -1, the new rule gets appended to the end.
  *
- * Return value: pointer to the newly created #Itdb_SPLRule. Its memory is handled 
- * by @pl though, so you don't need to explicitly call itdb_splr_free() on it
- **/
+ * Returns: pointer to the newly created #Itdb_SPLRule. Its
+ * memory is handled by @pl though, so you don't need to explicitly
+ * call itdb_splr_free() on it
+ */
 Itdb_SPLRule *itdb_splr_add_new (Itdb_Playlist *pl, gint pos)
 {
     Itdb_SPLRule *splr;
@@ -1126,18 +1140,18 @@ static Itdb_SPLRule *splr_duplicate (Itdb_SPLRule *splr)
     return dup;
 }
 
-
 /**
  * itdb_playlist_duplicate:
  * @pl: an #Itdb_Playlist
  *
- * Duplicates an existing playlist. pl_dup-&gt;id is set to zero, so that
- * it will be set to a unique value when adding it to an #Itdb_iTunesDB. The
- * returned playlist won't be associated with an #Itdb_iTunesDB.
+ * Duplicates an existing playlist. @pl_dup->id is set to zero, so
+ * that it will be set to a unique value when adding it to an
+ * #Itdb_iTunesDB. The returned playlist won't be associated with an
+ * #Itdb_iTunesDB.
  *
- * Return value: a newly allocated #Itdb_Playlist that you'll have to free
- * with itdb_playlist_free() when you no longer need it. 
- **/
+ * Returns: a newly allocated #Itdb_Playlist that you'll have to
+ * free with itdb_playlist_free() when you no longer need it.
+ */
 Itdb_Playlist *itdb_playlist_duplicate (Itdb_Playlist *pl)
 {
     Itdb_Playlist *pl_dup;
@@ -1176,19 +1190,21 @@ Itdb_Playlist *itdb_playlist_duplicate (Itdb_Playlist *pl)
     if (pl->userdata && pl->userdata_duplicate)
 	pl_dup->userdata = pl->userdata_duplicate (pl->userdata);
 
+    /* Copy private data too */
+    pl_dup->priv = g_memdup (pl->priv, sizeof (Itdb_Playlist_Private));
+
     return pl_dup;
 }
 
-
 /**
  * itdb_spl_copy_rules:
- * @dest: destination #Itdb_Playlist
- * @src: source #Itdb_Playlist
+ * @dest:   destination #Itdb_Playlist
+ * @src:    source #Itdb_Playlist
  *
  * Copy all relevant information for smart playlist from playlist @src
- * to playlist @dest. If @dest is already a smart playlist, the existing data
- * is overwritten/deleted. 
- **/
+ * to playlist @dest. If @dest is already a smart playlist, the
+ * existing data is overwritten/deleted.
+ */
 void itdb_spl_copy_rules (Itdb_Playlist *dest, Itdb_Playlist *src)
 {
     GList *gl;
@@ -1216,20 +1232,18 @@ void itdb_spl_copy_rules (Itdb_Playlist *dest, Itdb_Playlist *src)
     }
 }
 
-
-
 /**
  * itdb_playlist_new:
- * @title: playlist title
- * @spl: smart playlist flag
+ * @title:  playlist title
+ * @spl:    smart playlist flag
  *
- * Creates a new playlist. If @spl is TRUE, a smart
- * playlist is generated. pl->id is set by itdb_playlist_add() when the 
- * playlist is added to an #Itdb_iTunesDB
- * 
- * Return value: a new #Itdb_Playlist which must be freed with 
+ * Creates a new playlist. If @spl is TRUE, a smart playlist is
+ * generated. pl->id is set by itdb_playlist_add() when the playlist
+ * is added to an #Itdb_iTunesDB
+ *
+ * Returns: a new #Itdb_Playlist which must be freed with
  * itdb_playlist_free() after use
- **/
+ */
 Itdb_Playlist *itdb_playlist_new (const gchar *title, gboolean spl)
 {
     Itdb_Playlist *pl = g_new0 (Itdb_Playlist, 1);
@@ -1238,7 +1252,7 @@ Itdb_Playlist *itdb_playlist_new (const gchar *title, gboolean spl)
     pl->name = g_strdup (title);
     pl->sortorder = ITDB_PSO_MANUAL;
 
-    pl->timestamp = itdb_time_get_mac_time ();
+    pl->timestamp = time (NULL);
 
     pl->is_spl = spl;
     if (spl)
@@ -1254,16 +1268,17 @@ Itdb_Playlist *itdb_playlist_new (const gchar *title, gboolean spl)
 	/* add at least one rule */
 	itdb_splr_add_new (pl, 0);
     }
+    pl->priv = g_new0 (Itdb_Playlist_Private, 1);
+
     return pl;
 }
 
-
 /**
  * itdb_playlist_free:
  * @pl: an #Itdb_Playlist
  *
- * Frees the memory used by playlist @pl. 
- **/
+ * Frees the memory used by playlist @pl.
+ */
 void itdb_playlist_free (Itdb_Playlist *pl)
 {
     g_return_if_fail (pl);
@@ -1274,23 +1289,13 @@ void itdb_playlist_free (Itdb_Playlist *pl)
     g_list_free (pl->splrules.rules);
     if (pl->userdata && pl->userdata_destroy)
 	(*pl->userdata_destroy) (pl->userdata);
+
+    g_free (pl->priv);
     g_free (pl);
 }
 
-
-
-/**
- * itdb_playlist_add:
- * @itdb: an #Itdb_iTunesDB
- * @pl: an #Itdb_Playlist
- * @pos: position to insert @pl at
- *
- * Adds playlist @pl to the database @itdb at position @pos (-1 for
- * "append to end"). A unique id is created if pl-&gt;id is equal to
- * zero. After calling this function, @itdb manages the memory of @pl, which
- * means you no longer need to explicitly call itdb_playlist_free()
- **/
-void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
+static void itdb_playlist_add_internal (Itdb_iTunesDB *itdb, Itdb_Playlist *pl,
+                                        gint32 pos, GList **playlists)
 {
     g_return_if_fail (itdb);
     g_return_if_fail (pl);
@@ -1310,7 +1315,7 @@ void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
 	    /* check if id is really unique (with 100 playlists the
 	     * chance to create a duplicate is 1 in
 	     * 184,467,440,737,095,516.16) */
-	    for (gl=itdb->playlists; id && gl; gl=gl->next)
+	    for (gl=*playlists; id && gl; gl=gl->next)
 	    {
 		Itdb_Playlist *g_pl = gl->data;
 		g_return_if_fail (g_pl);
@@ -1320,22 +1325,44 @@ void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
 	pl->id = id;
     }
     if (pl->sortorder == 0)  pl->sortorder = ITDB_PSO_MANUAL;
-    if (pl->timestamp == 0)  pl->timestamp = itdb_time_get_mac_time ();
+    if (pl->timestamp == 0)  pl->timestamp = time (NULL);
 
     /* pos == -1 appends at the end of the list */
-    itdb->playlists = g_list_insert (itdb->playlists, pl, pos);
+    *playlists = g_list_insert (*playlists, pl, pos);
 }
 
+void itdb_playlist_add_mhsd5_playlist (Itdb_iTunesDB *itdb, Itdb_Playlist *pl,
+                                       gint32 pos)
+{
+    itdb_playlist_add_internal (itdb, pl, pos, &itdb->priv->mhsd5_playlists);
+}
 
+/**
+ * itdb_playlist_add:
+ * @itdb:   an #Itdb_iTunesDB
+ * @pl:     an #Itdb_Playlist
+ * @pos:    position to insert @pl at
+ *
+ * Adds playlist @pl to the database @itdb at position @pos (-1 for
+ * "append to end"). A unique id is created if @pl->id is equal to
+ * zero. After calling this function, @itdb manages the memory of @pl,
+ * which means you no longer need to explicitly call
+ * itdb_playlist_free()
+ */
+void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
+{
+    itdb_playlist_add_internal (itdb, pl, pos, &itdb->playlists);
+}
 
 /**
  * itdb_playlist_move:
- * @pl: an #Itdb_Playlist
- * @pos: new position
+ * @pl:     an #Itdb_Playlist
+ * @pos:    new position
  *
- * Moves playlist @pl to position @pos 
- **/
-void itdb_playlist_move (Itdb_Playlist *pl, guint32 pos)
+ * Moves playlist @pl to position @pos. -1 will append to the end of the
+ * list.
+ */
+void itdb_playlist_move (Itdb_Playlist *pl, gint32 pos)
 {
     Itdb_iTunesDB *itdb;
 
@@ -1347,14 +1374,13 @@ void itdb_playlist_move (Itdb_Playlist *pl, guint32 pos)
     itdb->playlists = g_list_insert (itdb->playlists, pl, pos);
 }
 
-
 /**
  * itdb_playlist_remove:
  * @pl: an #Itdb_Playlist
- * 
- * Removes @pl from the #Itdb_iTunesDB it's associated with 
- * and frees memory 
- **/
+ *
+ * Removes @pl from the #Itdb_iTunesDB it's associated with
+ * and frees memory
+ */
 void itdb_playlist_remove (Itdb_Playlist *pl)
 {
     Itdb_iTunesDB *itdb;
@@ -1367,14 +1393,13 @@ void itdb_playlist_remove (Itdb_Playlist *pl)
     itdb_playlist_free (pl);
 }
 
-
 /**
  * itdb_playlist_unlink:
  * @pl: an #Itdb_Playlist
  *
- * Remove @pl from the #Itdb_iTunesDB it's associated with but do not free 
- * memory. pl-&gt;itdb is set to NULL after this function returns
- **/
+ * Remove @pl from the #Itdb_iTunesDB it's associated with but do not
+ * free memory. @pl->itdb is set to NULL after this function returns
+ */
 void itdb_playlist_unlink (Itdb_Playlist *pl)
 {
     Itdb_iTunesDB *itdb;
@@ -1387,16 +1412,15 @@ void itdb_playlist_unlink (Itdb_Playlist *pl)
     pl->itdb = NULL;
 }
 
-
 /**
  * itdb_playlist_exists:
- * @itdb: an #Itdb_iTunesDB
- * @pl: an #Itdb_Playlist
+ * @itdb:   an #Itdb_iTunesDB
+ * @pl:     an #Itdb_Playlist
  *
- * Checks if @pl is present in @db
+ * Checks if @pl is present in @itdb
  *
- * Return value: TRUE if @pl exists in @db, FALSE otherwise 
- **/
+ * Returns: TRUE if @pl exists in @itdb, FALSE otherwise
+ */
 gboolean itdb_playlist_exists (Itdb_iTunesDB *itdb, Itdb_Playlist *pl)
 {
     g_return_val_if_fail (itdb, FALSE);
@@ -1406,16 +1430,14 @@ gboolean itdb_playlist_exists (Itdb_iTunesDB *itdb, Itdb_Playlist *pl)
     else                                    return FALSE;
 }
 
-
 /**
  * itdb_playlist_add_track:
- * @pl: an #Itdb_Playlist
- * @track: an #Itdb_Track
- * @pos: position to insert @track at
+ * @pl:     an #Itdb_Playlist
+ * @track:  an #Itdb_Track
+ * @pos:    position to insert @track at
  *
- * Adds @track to @pl at position @pos (-1 for "append to
- * end") 
- **/
+ * Adds @track to @pl at position @pos (-1 to append at the end)
+ */
 void itdb_playlist_add_track (Itdb_Playlist *pl,
 			      Itdb_Track *track, gint32 pos)
 {
@@ -1428,18 +1450,16 @@ void itdb_playlist_add_track (Itdb_Playlist *pl,
     pl->members = g_list_insert (pl->members, track, pos);
 }
 
-
-
 /**
  * itdb_playlist_remove_track:
- * @pl: an #Itdb_Playlist
- * @track: an #Itdb_Track
+ * @pl:     an #Itdb_Playlist
+ * @track:  an #Itdb_Track
  *
  * Removes @track from @pl. If @pl is NULL, removes @track from the
- * master playlist. If @track can't be found in @pl, nothing happens. If after
- * removing @track, @pl is empty, it's not removed from the database
- * The memory used by @track isn't freed.
- **/
+ * master playlist. If @track can't be found in @pl, nothing happens.
+ * If after removing @track, @pl is empty, it's not removed from the
+ * database The memory used by @track isn't freed.
+ */
 void itdb_playlist_remove_track (Itdb_Playlist *pl, Itdb_Track *track)
 {
     g_return_if_fail (track);
@@ -1452,17 +1472,16 @@ void itdb_playlist_remove_track (Itdb_Playlist *pl, Itdb_Track *track)
     pl->members = g_list_remove (pl->members, track);
 }
 
-
 /**
  * itdb_playlist_by_id:
- * @itdb: an #Itdb_iTunesDB
- * @id: ID of the playlist to look for
+ * @itdb:   an #Itdb_iTunesDB
+ * @id:     ID of the playlist to look for
  *
- * Looks up a playlist whose ID is @id 
+ * Looks up a playlist whose ID is @id
  *
- * Return value: the #Itdb_Playlist with ID @id or NULL if there is no such
- * playlist.
- **/
+ * Returns: the #Itdb_Playlist with ID @id or NULL if there is no
+ * such playlist.
+ */
 Itdb_Playlist *itdb_playlist_by_id (Itdb_iTunesDB *itdb, guint64 id)
 {
     GList *gl;
@@ -1477,16 +1496,16 @@ Itdb_Playlist *itdb_playlist_by_id (Itdb_iTunesDB *itdb, guint64 id)
     return NULL;
 }
 
-
 /**
  * itdb_playlist_by_nr:
- * @itdb: an #Itdb_iTunesDB
- * @num: the position of the playlist, counting from 0
+ * @itdb:   an #Itdb_iTunesDB
+ * @num:    the position of the playlist, counting from 0
  *
  * Gets the playlist at the given position in @itdb
  *
- * Return value: the #Itdb_Playlist, or NULL if there is no playlist at @pos
- **/
+ * Returns: the #Itdb_Playlist, or NULL if there is no playlist
+ * at @pos
+ */
 Itdb_Playlist *itdb_playlist_by_nr (Itdb_iTunesDB *itdb, guint32 num)
 {
     Itdb_Playlist *pl;
@@ -1496,17 +1515,16 @@ Itdb_Playlist *itdb_playlist_by_nr (Itdb_iTunesDB *itdb, guint32 num)
     return pl;
 }
 
-
 /**
  * itdb_playlist_by_name:
- * @itdb: an #Itdb_iTunesDB
- * @name: name of the playlist to look for
+ * @itdb:   an #Itdb_iTunesDB
+ * @name:   name of the playlist to look for
  *
  * Searches a playlist whose name is @name in @itdb
  *
- * Return value: the first #Itdb_Playlist with name @name, NULL if there is no
- * such playlist
- **/
+ * Returns: the first #Itdb_Playlist with name @name, NULL if
+ * there is no such playlist
+ */
 Itdb_Playlist *itdb_playlist_by_name (Itdb_iTunesDB *itdb, gchar *name)
 {
     GList *gl;
@@ -1523,15 +1541,16 @@ Itdb_Playlist *itdb_playlist_by_name (Itdb_iTunesDB *itdb, gchar *name)
     return NULL;
 }
 
-
 /**
  * itdb_playlist_is_mpl:
  * @pl: an #Itdb_Playlist
  *
- * Checks if @pl is the master playlist 
+ * Checks if @pl is the master playlist
  *
- * Return value: TRUE if @pl is the master playlist, FALSE otherwise 
- **/
+ * Returns: TRUE if @pl is the master playlist, FALSE otherwise
+ *
+ * Since: 0.1.6
+ */
 gboolean itdb_playlist_is_mpl (Itdb_Playlist *pl)
 {
     g_return_val_if_fail (pl, FALSE);
@@ -1539,15 +1558,16 @@ gboolean itdb_playlist_is_mpl (Itdb_Playlist *pl)
     return ((pl->type & 0xff) == ITDB_PL_TYPE_MPL);
 }
 
-
 /**
  * itdb_playlist_is_podcasts:
  * @pl: an #Itdb_Playlist
  *
  * Checks if @pl is the podcasts playlist
  *
- * Return value: TRUE if @pl is the podcasts playlist, FALSE otherwise 
- **/
+ * Returns: TRUE if @pl is the podcasts playlist, FALSE otherwise
+ *
+ * Since: 0.1.6
+ */
 gboolean itdb_playlist_is_podcasts (Itdb_Playlist *pl)
 {
     g_return_val_if_fail (pl, FALSE);
@@ -1555,13 +1575,14 @@ gboolean itdb_playlist_is_podcasts (Itdb_Playlist *pl)
     return (pl->podcastflag == ITDB_PL_FLAG_PODCASTS);
 }
 
-
 /**
  * itdb_playlist_set_mpl:
  * @pl: an #Itdb_Playlist
  *
  * Sets @pl to be a master playlist
- **/
+ *
+ * Since: 0.2.0
+ */
 void itdb_playlist_set_mpl (Itdb_Playlist *pl)
 {
     g_return_if_fail (pl);
@@ -1569,13 +1590,14 @@ void itdb_playlist_set_mpl (Itdb_Playlist *pl)
     pl->type = ITDB_PL_TYPE_MPL;
 }
 
-
 /**
  * itdb_playlist_set_podcasts:
  * @pl: an #Itdb_Playlist
  *
- * Set @pl to be a podcasts playlist 
- **/
+ * Set @pl to be a podcasts playlist
+ *
+ * Since: 0.2.0
+ */
 void itdb_playlist_set_podcasts (Itdb_Playlist *pl)
 {
     g_return_if_fail (pl);
@@ -1583,15 +1605,14 @@ void itdb_playlist_set_podcasts (Itdb_Playlist *pl)
     pl->podcastflag = ITDB_PL_FLAG_PODCASTS;
 }
 
-
 /**
  * itdb_playlist_mpl:
  * @itdb: an #Itdb_iTunesDB
- * 
+ *
  * Gets the master playlist of @itdb
  *
- * Return value: the master playlist of @itdb 
- **/
+ * Returns: the master playlist of @itdb
+ */
 Itdb_Playlist *itdb_playlist_mpl (Itdb_iTunesDB *itdb)
 {
     Itdb_Playlist *pl;
@@ -1607,15 +1628,17 @@ Itdb_Playlist *itdb_playlist_mpl (Itdb_iTunesDB *itdb)
     return pl;
 }
 
-
 /**
  * itdb_playlist_podcasts:
  * @itdb: an #Itdb_iTunesDB
  *
  * Gets the podcasts playlist of @itdb
  *
- * Return value: the podcasts playlist of @itdb, or NULL if it's there is none
- **/
+ * Returns: the podcasts playlist of @itdb, or NULL if there is
+ * not one
+ *
+ * Since: 0.1.6
+ */
 Itdb_Playlist *itdb_playlist_podcasts (Itdb_iTunesDB *itdb)
 {
     GList *gl;
@@ -1633,17 +1656,15 @@ Itdb_Playlist *itdb_playlist_podcasts (Itdb_iTunesDB *itdb)
     return NULL;
 }
 
-
-
 /**
  * itdb_playlist_contains_track:
- * @pl: an #Itdb_Playlist
- * @track: an #Itdb_Track
+ * @pl:     an #Itdb_Playlist
+ * @track:  an #Itdb_Track
  *
- * Checks if @track is in @pl.
- * 
- * Return value: TRUE if @track is in @pl, FALSE otherwise
- **/
+ * Checks if @track is in @pl
+ *
+ * Returns: TRUE if @track is in @pl, FALSE otherwise
+ */
 gboolean itdb_playlist_contains_track (Itdb_Playlist *pl, Itdb_Track *tr)
 {
     g_return_val_if_fail (tr, FALSE);
@@ -1657,16 +1678,15 @@ gboolean itdb_playlist_contains_track (Itdb_Playlist *pl, Itdb_Track *tr)
     else                                return FALSE;
 }
 
-
 /**
  * itdb_playlist_contain_track_number:
  * @tr: an #Itdb_Track
  *
- * Counts the number of playlist @track is a member of (not including the 
- * master playlist)
+ * Counts the number of playlist @track is a member of (not including
+ * the master playlist)
  *
- * Return value: number of playlist containing @track
- **/
+ * Returns: the number of playlist containing @track
+ */
 guint32 itdb_playlist_contain_track_number (Itdb_Track *tr)
 {
     Itdb_iTunesDB *itdb;
@@ -1688,16 +1708,14 @@ guint32 itdb_playlist_contain_track_number (Itdb_Track *tr)
     return num;
 }
 
-
-
 /**
  * itdb_playlist_tracks_number:
  * @pl: an #Itdb_Playlist
  *
  * Counts the number of tracks in @pl
  *
- * Return value: track count
- **/
+ * Returns: the number of tracks in @pl
+ */
 guint32 itdb_playlist_tracks_number (Itdb_Playlist *pl)
 {
     g_return_val_if_fail (pl, 0);
