@@ -107,6 +107,7 @@ struct _SysInfoIpodProperties {
         gint games_platform_id;
         gint games_platform_version;
         gint rental_clock_bias;
+	gboolean sqlite_db;
 };
 
 static gint get_int (GHashTable *dict, const char *key)
@@ -216,6 +217,7 @@ static const DictFieldMapping sysinfo_ipod_properties_fields_mapping[] = {
     { "GamesPlatformID",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_id) },
     { "GamesPlatformVersion",          G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_version) },
     { "RentalClockBias",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, rental_clock_bias) },
+    { "SQLiteDB",                      G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, sqlite_db) },
     { NULL,                            G_TYPE_NONE,    0 }
 };
 
@@ -368,6 +370,7 @@ static void dump_struct (const DictFieldMapping *mapping,
 static void dump_image_format (Itdb_ArtworkFormat *format)
 {
     dump_struct (sysinfo_image_format_fields_mapping, format);
+    g_print ("PixelFormat: %d\n", format->format); 
 }
 
 void itdb_sysinfo_properties_dump (SysInfoIpodProperties *props)
@@ -396,11 +399,18 @@ set_pixel_format (Itdb_ArtworkFormat *img_spec, GHashTable *dict)
         img_spec->format = THUMB_FORMAT_RGB565_LE;
     } else if (strcmp (pixel_format, "79343230" /* y420 */) == 0) {
         img_spec->format = THUMB_FORMAT_I420_LE;
+    } else if (strcmp (pixel_format, "4C353535" /* L555 */) == 0) {
+	if (g_hash_table_lookup (dict, "PixelOrder") != NULL) {
+	    img_spec->format = THUMB_FORMAT_REC_RGB555_LE;
+	} else {
+	    img_spec->format = THUMB_FORMAT_RGB555_LE;
+	}
     } else {
         g_free (pixel_format);
         return FALSE;
     }
     g_hash_table_remove (dict, "PixelFormat");
+    g_hash_table_remove (dict, "PixelOrder");
     g_free (pixel_format);
     return TRUE;
 }
@@ -468,6 +478,12 @@ static GList *parse_one_formats_list (GHashTable *sysinfo_dict,
     array = (GValueArray*)g_value_get_boxed (to_parse);
     for (i = 0; i < array->n_values; i++) {
         Itdb_ArtworkFormat *format;
+	/* SysInfoExtended on the iPhone has <string> fields in the artwork
+	 * format array in addition to the hash we parse
+	 */
+	if (!G_VALUE_HOLDS (g_value_array_get_nth (array, i), G_TYPE_HASH_TABLE)) {
+	    continue;
+	}
 	format = g_value_to_image_format (g_value_array_get_nth (array, i));
 	if (format != NULL) {
 		formats = g_list_prepend (formats, format);
@@ -532,6 +548,25 @@ SysInfoIpodProperties *itdb_sysinfo_extended_parse (const char *filename,
     return props;
 }
 
+SysInfoIpodProperties *itdb_sysinfo_extended_parse_from_xml (const char *xml,
+							     GError **error)
+{
+    GValue *parsed_doc;
+    SysInfoIpodProperties *props;
+
+    g_return_val_if_fail (xml != NULL, NULL);
+
+    parsed_doc = itdb_plist_parse_from_memory (xml, strlen (xml), error);
+    if (parsed_doc == NULL) {
+        return NULL;
+    }
+    props = g_value_to_ipod_properties (parsed_doc);
+    g_value_unset (parsed_doc);
+    g_free (parsed_doc);
+
+    return props;
+}
+
 /**
  * itdb_sysinfo_properties_get_serial_number:
  * @props: a #SysInfoIpodProperties structure
@@ -632,10 +667,48 @@ itdb_sysinfo_properties_get_chapter_image_formats (const SysInfoIpodProperties *
  * Returns: TRUE if the iPod supports sparse artwork, FALSE if it does not
  * or if @props doesn't contain any information about sparse artwork
  */
-G_GNUC_INTERNAL gboolean
+gboolean
 itdb_sysinfo_properties_supports_sparse_artwork (const SysInfoIpodProperties *props)
 {
     g_return_val_if_fail (props != NULL, FALSE);
 
     return props->supports_sparse_artwork;
 }
+
+gboolean
+itdb_sysinfo_properties_supports_podcast (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+
+    return props->podcasts_supported;
+}
+
+const char *
+itdb_sysinfo_properties_get_firmware_version (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, NULL);
+
+    return props->visible_build_id;
+}
+
+gboolean
+itdb_sysinfo_properties_supports_sqlite (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+
+    return props->sqlite_db;
+}
+
+gint
+itdb_sysinfo_properties_get_family_id (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+    return props->family_id;
+}
+
+gint
+itdb_sysinfo_properties_get_db_version (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+    return props->db_version;
+}
